init python:
    import pygame
    from pygame.locals import *
    import math  # Importar math correctamente

    # Configuración de la ventana de PyGame
    win_width, win_height = (1920, 1080)  # Tamaño más pequeño para integrar en Ren'Py
    move_speed = 0.01
    mouse_sensitivity = 0.002

    environment = [
        [1, 1, 1, 1, 1],
        [1, 0, 0, 0, 1],
        [1, 0, 1, 0, 1],
        [1, 0, 0, 0, 1],
        [1, 1, 1, 1, 1]
    ]
    # Función para mover al jugador
    def move_player(keys, xpos, ypos, rot_r):
        if keys[pygame.K_w]:  # Mover hacia adelante
            xpos += move_speed * math.cos(rot_r)
            ypos += move_speed * math.sin(rot_r)
        if keys[pygame.K_s]:  # Mover hacia atrás
            xpos -= move_speed * math.cos(rot_r)
            ypos -= move_speed * math.sin(rot_r)
        if keys[pygame.K_a]:  # Mover a la izquierda
            xpos += move_speed * math.sin(rot_r)
            ypos -= move_speed * math.cos(rot_r)
        if keys[pygame.K_d]:  # Mover a la derecha
            xpos -= move_speed * math.sin(rot_r)
            ypos += move_speed * math.cos(rot_r)
        return xpos, ypos

    # Función para rotar la cámara con el mouse
    def rotate_camera(mouse_rel, rot_r):
        rot_r += mouse_rel[0] * mouse_sensitivity
        return rot_r

    # Función para calcular el color de la pared basado en el ángulo
    def get_wall_color(angle_diff):
        brightness = max(0, 1 - (abs(angle_diff) / (math.pi / 2)) ** 6)
        brightness = brightness ** 0.5
        return (int(255 * brightness), int(128 * brightness), 0)  # Paredes naranjas con sombreado

    # Función principal de raycasting
    def raycasting_main():
        global xpos, ypos, rot_r

        # Inicializar PyGame
        pygame.init()
        display = pygame.display.set_mode((win_width, win_height))
        pygame.display.set_caption("Raycasting en Ren'Py")

        # Configuración de la cámara
        fov = 80
        xpos, ypos = (1.5, 1.5)  # Posición inicial del jugador
        rot_r = math.pi / 4  # Rotación inicial (usar math.pi en lugar de m.pi)

        # Ocultar el cursor del mouse
        pygame.mouse.set_visible(False)
        pygame.event.set_grab(True)

        # Bucle principal de PyGame
        run = True
        while run:
            for e in pygame.event.get():
                if e.type == QUIT:
                    run = False

            # Obtener las teclas presionadas
            keys = pygame.key.get_pressed()

            # Obtener el movimiento relativo del mouse
            mouse_rel = pygame.mouse.get_rel()

            # Mover al jugador
            xpos, ypos = move_player(keys, xpos, ypos, rot_r)

            # Rotar la cámara con el mouse
            rot_r = rotate_camera(mouse_rel, rot_r)

            # Limpiar la pantalla
            display.fill((0, 0, 0))

            # Raycasting
            for i in range(fov):
                rot_d = rot_r + math.radians(i - fov / 2)
                x, y = (xpos, ypos)
                sin, cos = (0.02 * math.sin(rot_d), 0.02 * math.cos(rot_d))
                j = 0
                while True:
                    x, y = (x + cos, y + sin)
                    j += 1
                    if environment[int(x)][int(y)] != 0:
                        height = (5 / j * 2500)
                        break

                # Calcular la diferencia de ángulo entre la pared y la cámara
                angle_diff = rot_d - rot_r
                angle_diff = (angle_diff + math.pi) % (2 * math.pi) - math.pi  # Normalizar entre -π y π

                # Obtener el color de la pared basado en el ángulo
                wall_color = get_wall_color(angle_diff)

                # Dibujar la línea con el color calculado
                pygame.draw.line(display, wall_color, (i * (win_width / fov), win_height / 2 + height), (i * (win_width / fov), win_height / 2 - height), int(win_width / fov) // 2)

            # Actualizar la pantalla
            pygame.display.update()

            # Salir si se presiona ESC
            if keys[pygame.K_ESCAPE]:
                run = False

        # Limpiar PyGame al salir
        pygame.quit()

# Pantalla de Ren'Py para ejecutar el raycasting
screen raycasting_screen():
    # Ejecutar el raycasting en una ventana de PyGame
    button:
        action Function(raycasting_main)
        text "Iniciar Raycasting"

# Inicio del juego
label starts:
    "Bienvenido al juego."
    call screen raycasting_screen
    "Has salido del modo raycasting."
    return